@@ -4,10 +4,10 @@ import logging
 import sys
 import warnings
 import multiprocessing as mp
+import random
 import networkx as nx
 import numpy as np
 import pandas as pd
-import random
 from deap import base, creator, tools
 
 from .utils import chunk_it
@@ -63,7 +63,8 @@ class FaultDiagnosis():
         cols_to_str = ['init_status']
         gephi_nodes_df[cols_to_str] = gephi_nodes_df[cols_to_str].astype(str)
         conversions = {'nan': '', '1.0': '1', '0.0': '0'}
-        gephi_nodes_df['init_status'].replace(to_replace=conversions, inplace=True)
+        gephi_nodes_df['init_status'].replace(to_replace=conversions,
+            inplace=True)
 
         gephi_nodes_df[fields].to_csv('check_import_nodes.csv', index=False)
 
@@ -72,7 +73,7 @@ class FaultDiagnosis():
         self.edges_df.to_csv('check_import_edges.csv', index=False)
 
     def fitness_iteration_parallel(self, out_queue, ichunk, chunk_length,
-        individuals, perturbed_nodes, initial_condition, weights):
+        individuals, perturbed_nodes, initial_condition, origin):
         """
 
         Parallel iteration for fitness evaluation. We append to the
@@ -80,48 +81,43 @@ class FaultDiagnosis():
         index of the individual, the individual itself, and its fitness.
 
         :param multiprocessing.queues.Queue out_queue: multiprocessing queue
-        :param int ichunk: index of the chunk under consideration
+        :param int ichunk: index of the chunk under consideration.
         :param int chunk_length: lengths of the chunks (the last chunk may
             be shorter due to non-even division of the number of generations by
-            the number of processors)
+            the number of processors).
         :param list individuals: list of individuals on which to perform
             fitness evaluation.
         :param list perturbed_nodes: nodes(s) involved in the perturbing event.
         :param dict initial_condition: initial status (boolean) for the graph
             switches.
-        :param dict weights: weights for fitness evaluation on individuals.
-            Dict of: {str: float, str: float, str: float}:
-            - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
-
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
         """
 
-        for iter_ind in range(len(individuals)):
-            ind_fit = (ichunk*chunk_length + iter_ind, individuals[iter_ind],
-                self.fitness_evaluation(individuals[iter_ind], perturbed_nodes,
-                initial_condition, weights))
+        for idx, item in enumerate(individuals):
+            ind_fit = (ichunk*chunk_length + idx, item,
+                self.fitness_evaluation(item, perturbed_nodes,
+                initial_condition, origin))
             out_queue.put(ind_fit)
 
     def fitness_evaluation_parallel(self, pop, perturbed_nodes,
-        initial_condition, weights):
+        initial_condition, origin):
         """
 
         Wrapper for fitness evaluation. This methods spawns the processes for
         fitness evaluation and collects the results.
 
-        :param list pop: list of individuals
+        :param list pop: list of individuals.
         :param list perturbed_nodes: nodes(s) involved in the perturbing event.
         :param dict initial_condition: initial status (boolean) for the graph
             switches.
-        :param dict weights: weights for fitness evaluation on individuals.
-            Dict of: {str: float, str: float, str: float}:
-            - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
 
         :return: list of tuples constituted by the index of the individual,
-            the individual itself, and its fitness
+            the individual itself, and its fitness.
         :rtype: list
         """
 
@@ -134,7 +130,7 @@ class FaultDiagnosis():
         processes = [
             mp.Process( target=self.fitness_iteration_parallel,
             args=( out_queue, p, len(ind_chunks[0]), ind_chunks[p],
-                perturbed_nodes, initial_condition, weights ))
+                perturbed_nodes, initial_condition, origin ))
                 for p in range(n_procs) ]
 
         for proc in processes:
@@ -151,8 +147,8 @@ class FaultDiagnosis():
 
         return fitnesses_tuples
 
-    def fitness_evaluation(self, individual, perturbed_nodes, initial_condition,
-        w):
+    def fitness_evaluation(self, individual, perturbed_nodes,
+        initial_condition, origin):
         """
 
         Evaluation of fitness on individual.
@@ -166,18 +162,16 @@ class FaultDiagnosis():
             perturbing event.
         :param dict initial_condition: initial status (boolean) for the graph
             switches.
-        :param dict w: weights for fitness evaluation on individuals.
-            Dict of: {str: float, str: float, str: float}:
-            - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
         """
 
         acts = np.sum(np.not_equal(list(initial_condition.values()),
             individual))
 
         T = GeneralGraph()
-        T.add_nodes_from(self.G) 
+        T.add_nodes_from(self.G)
         for (u, v, d) in self.G.edges(data=True):
             T.add_edge(u, v, father_condition=d['father_condition'],
                 weight=d['weight'])
@@ -195,16 +189,27 @@ class FaultDiagnosis():
         for node in perturbed_nodes:
             if node in T.nodes():
 
-                _, broken_nodes = self.rm_nodes(node, T)
+                _, broken_nodes = self.rm_nodes(node, T, origin)
                 broken_nodes = list(set(broken_nodes))
 
-                for n in broken_nodes: T.remove_node(n)
+                for n in broken_nodes:
+                    T.remove_node(n)
 
-        fit = w['w1']*acts - w['w2']*sum(T.service.values()) - w['w3']*len(T)
-        return (fit,)
+        serv_at_usr = {key: T.service[key] for key in T.users}
+        usr_with_serv = {key: serv_at_usr[key] for key in serv_at_usr.keys()
+            if serv_at_usr[key] > 0.0}
+
+        avg_service = sum(T.service.values())
+        if len(usr_with_serv.keys()) != 0:
+            avg_service /= len(usr_with_serv.keys())
+        dist_from_avg =  {k: abs(v - avg_service)
+            for k, v in usr_with_serv.items()}
+
+        return (acts, sum(T.service.values()), len(T),
+            len(usr_with_serv.keys()), sum(dist_from_avg.values()))
 
     def optimizer(self, perturbed_nodes, initial_condition, params, weights,
-        parallel):
+        parallel, origin):
         """
 
         Genetic algorithm to optimize switches conditions, using DEAP.
@@ -224,17 +229,24 @@ class FaultDiagnosis():
         :param dict weights: weights for fitness evaluation on individuals.
             Dict of: {str: float, str: float, str: float}:
             - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+            - 'w2': weight multiplying total final service (default to -1.0)
+            - 'w3': weight multiplying final graph size (default to -1.0)
+            - 'w4': weight multiplying number of users with non-zero service
+                (default to -1.0)
+            - 'w5': weight for service balance over users (default to 2.0)
         :param bool parallel: flag for parallel fitness evaluation of
-            initial population
+            initial populations.
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
         """
 
         logging.getLogger().setLevel(logging.INFO)
+        w = np.asarray(list(weights.values()))
 
         creator.create("FitnessMin", base.Fitness, weights=(-1.0,))
         creator.create("Individual", list, fitness=creator.FitnessMin)
-    
+
         toolbox = base.Toolbox()
         # Attribute generator
         toolbox.register("attribute_bool", random.choice, [True, False])
@@ -243,68 +255,84 @@ class FaultDiagnosis():
             toolbox.attribute_bool, len(self.G.switches))
         toolbox.register("population", tools.initRepeat, list,
             toolbox.individual)
-    
+
         toolbox.register("evaluate", self.fitness_evaluation)
         toolbox.register("mate", tools.cxUniform, indpb=params['indpb'])
-        toolbox.register("mutate", tools.mutShuffleIndexes,
-            indpb=params['indpb'])
-        toolbox.register("select", tools.selBest)
+        toolbox.register("mutate", tools.mutFlipBit, indpb=params['indpb'])
 
         pop = toolbox.population(n=params['npop'])
+
         # Evaluate the entire population
-        if not parallel:
+        if (not parallel) or (len(pop) < mp.cpu_count()):
             fitnesses = [toolbox.evaluate(ind, perturbed_nodes,
-                initial_condition, weights) for ind in pop]
+                initial_condition, origin) for ind in pop]
         else:
             res_par = self.fitness_evaluation_parallel(pop, perturbed_nodes,
-                initial_condition, weights)
+                initial_condition, origin)
             res_par.sort(key=lambda x:x[0])
             fitnesses = [x[2] for x in res_par]
 
-        for ind, fit in zip(pop, fitnesses):
-            ind.fitness.values = fit
-        fits = [ind.fitness.values[0] for ind in pop]
+        for ind, f in zip(pop, fitnesses):
+            ind.fitness.values = (np.dot(w, np.asarray(f)),)
 
-        # Variable keeping track of the number of generations
+        # Generations
         g = 0
-    
-        fitnesses = fits
         result = []
-        # Begin the evolution
+
+        pop = [list(couple) for couple in zip(pop, fitnesses)]
+        pop.sort(key=lambda x: np.dot(w, np.asarray(x[1])))
 
         while g < params['ngen']:
-            # A new generation
+
             g = g + 1
 
-            # Select the next generation individuals
-            offspring = toolbox.select(pop, params['nsel'])
-            # Clone the selected individuals
-            offspring = list(map(toolbox.clone, offspring))
+            # Select next generation
+            offspring = [x[0] for x in pop[:params['nsel']]]
+            pop = pop[:params['nsel']]
 
-            # Apply crossover and mutation on the offspring
-            for child1, child2 in zip(offspring[::2], offspring[1::2]):
+            invalid_ind = []
+
+            # Crossover and mutation on the offspring
+            for parent1, parent2 in zip(offspring[::2], offspring[1::2]):
                 if random.random() < params['tresh']:
+                    child1 = toolbox.clone(parent1)
+                    child2 = toolbox.clone(parent2)
                     toolbox.mate(child1, child2)
                     del child1.fitness.values
+                    invalid_ind.append(child1)
                     del child2.fitness.values
-    
-            for mutant in offspring:
+                    invalid_ind.append(child2)
+
+            for ind in offspring:
                 if random.random() < params['tresh']:
+                    mutant = toolbox.clone(ind)
                     toolbox.mutate(mutant)
                     del mutant.fitness.values
-    
+                    invalid_ind.append(mutant)
+
             # Evaluate the individuals with an invalid fitness
-            invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
-            fitnesses = [toolbox.evaluate(ind, perturbed_nodes,
-                initial_condition, weights) for ind in invalid_ind]
+            if (not parallel) or (len(invalid_ind) < mp.cpu_count()):
+                fitnesses = [toolbox.evaluate(ind, perturbed_nodes,
+                    initial_condition, origin) for ind in invalid_ind]
+            else:
+                res_par = self.fitness_evaluation_parallel(invalid_ind,
+                    perturbed_nodes, initial_condition, origin)
+                res_par.sort(key=lambda x:x[0])
+                fitnesses = [x[2] for x in res_par]
 
-            for ind, fit in zip(invalid_ind, list(fitnesses)):
-                ind.fitness.values = fit
+            for ind, f in zip(invalid_ind, list(fitnesses)):
+                ind.fitness.values = (np.dot(w, np.asarray(f)),)
 
-            pop[:] = offspring[:] + invalid_ind[:]
+            ind_fit_invalid = [list(couple) for couple in zip(invalid_ind,
+                fitnesses)]
 
-            best = toolbox.select(pop, 1)
-            result.append([best[0], best[0].fitness.values[0]])
+            pop[:] = pop[:] + ind_fit_invalid[:]
+
+            # Sort the population according to fitness
+            pop.sort(key=lambda x: np.dot(w, np.asarray(x[1])))
+
+            # Extracting best state
+            result.append(pop[0])
 
         logging.getLogger().setLevel(logging.DEBUG)
 
@@ -369,7 +397,7 @@ class FaultDiagnosis():
                 self.paths_df = pd.merge(self.paths_df, df,
                     on=['from', 'to', 'ids', 'area'], how='outer')
 
-    def rm_nodes(self, node, graph, visited=None, broken_nodes=None):
+    def rm_nodes(self, node, graph, origin, visited=None, broken_nodes=None):
         """
 
         Remove nodes from the graph in a depth first search way to
@@ -381,66 +409,68 @@ class FaultDiagnosis():
 
         :param str node: the id of the node to remove.
         :param nx.DiGraph graph: graph on which to apply the node deletion
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
         :param visited: nodes already visited, default to None.
         :type visited: set, optional
         :param broken_nodes: nodes that got broken along the perturbation,
             default to None.
-        :type visited: list, optional
+        :type broken_nodes: list, optional
         """
 
         if visited is None:
             visited = set()
             broken_nodes = []
         visited.add(node)
-        logging.debug(f'Visited: {visited}')
-        logging.debug(f'Node: {node}')
+        logging.debug('Visited: %s', visited)
+        logging.debug('Node: %s', node)
 
-
-        if bool(graph.perturbation_resistant[node]):
-            logging.debug(f'Node {node} visited, fault resistant node')
+        if origin in graph.perturbation_resistant[node]:
+            logging.debug('Node %s visited, fault resistant node', node)
+            logging.debug('Perturbation origin: %s', origin)
             return visited, broken_nodes
 
-        else:
-            fathers = {'AND': set(), 'OR': set(), 'SINGLE': set() }
-            predecessors = list(graph.predecessors(node))
-            logging.debug(f'Predecessors: {predecessors}')
+        fathers = {'AND': set(), 'OR': set(), 'SINGLE': set() }
+        predecessors = list(graph.predecessors(node))
+        logging.debug('Predecessors: %s', predecessors)
 
-            if len(visited) == 1:
+        if len(visited) == 1:
+            broken_nodes.append(node)
+            logging.debug('Broken: %s', broken_nodes)
+
+        elif predecessors:
+            for p in predecessors:
+                fathers[graph.father_condition[(p, node)]].add(p)
+
+            if fathers['AND'] & set(broken_nodes):
                 broken_nodes.append(node)
-                logging.debug(f'Broken: {broken_nodes}')
+                logging.debug('Broken %s, AND predecessor broken.', node)
+                logging.debug('Nodes broken so far: %s', broken_nodes)
 
-            elif predecessors:
-                for p in predecessors:
-                    fathers[graph.father_condition[(p, node)]].add(p)
+            #'SINGLE' treated as 'AND'
+            elif fathers['SINGLE'] & set(broken_nodes):
+                broken_nodes.append(node)
+                logging.debug('Broken %s, SINGLE predecessor broken.', node)
+                logging.debug('Nodes broken so far: %s', broken_nodes)
 
-                if fathers['AND'] & set(broken_nodes):
-                    broken_nodes.append(node)
-                    logging.debug(f'Broken {node}, AND predecessor broken.')
-                    logging.debug(f'Nodes broken so far: {broken_nodes}')
-
-                #'SINGLE' treated as 'AND'
-                elif fathers['SINGLE'] & set(broken_nodes):
-                    broken_nodes.append(node)
-                    logging.debug(f'Broken {node}, SINGLE predecessor broken.')
-                    logging.debug(f'Nodes broken so far: {broken_nodes}')
-
-                else:
-                    if (fathers['OR'] & set(broken_nodes)) == set(predecessors):
-                        #all my 'OR' predecessors are dead
-                        broken_nodes.append(node)
-                        logging.debug(f'Broken {node}, no more fathers')
-                        logging.debug(f'Nodes broken so far: {broken_nodes}')
-                    else:
-                        logging.debug(f'Surviving fathers: {fathers}')
-                        logging.debug(f'Nodes broken so far: {broken_nodes}')
-                        return 0
             else:
-                broken_nodes.append(node)
-                logging.debug(f'Node: {node} has no more predecessors')
-                logging.debug(f'Nodes broken so far: {broken_nodes}')
+                if (fathers['OR'] & set(broken_nodes)) == set(predecessors):
+                    #all my 'OR' predecessors are dead
+                    broken_nodes.append(node)
+                    logging.debug('Broken %s, no more fathers', node)
+                    logging.debug('Nodes broken so far: %s', broken_nodes)
+                else:
+                    logging.debug('Surviving fathers: %s', fathers)
+                    logging.debug('Nodes broken so far: %s', broken_nodes)
+                    return 0
+        else:
+            broken_nodes.append(node)
+            logging.debug('Node: %s has no more predecessors', node)
+            logging.debug('Nodes broken so far: %s', broken_nodes)
 
         for next_node in set(graph[node]) - visited:
-            self.rm_nodes(next_node, graph, visited, broken_nodes)
+            self.rm_nodes(next_node, graph, origin, visited, broken_nodes)
 
         return visited, broken_nodes
 
@@ -482,26 +512,35 @@ class FaultDiagnosis():
         for area in damaged_areas:
             self.df.loc[self.df.area == area, 'status_area'] = 'DAMAGED'
 
-    def delete_a_node(self, node):
+    def delete_a_node(self, node, origin):
         """
 
         Delete a node in the graph.
 
         :param str node: the id of the node to remove.
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
 
         .. warning:: the node id must be contained in the graph.
             No check is done within this function.
         """
 
-        _ , broken_nodes = self.rm_nodes(node, self.G)
+        _ , broken_nodes = self.rm_nodes(node, self.G, origin)
         broken_nodes = list(set(broken_nodes))
+        self.G._del_nodes.extend(broken_nodes)
+
+        self.G._del_edges.extend(list(self.G.in_edges(nbunch=broken_nodes,
+            data=True)))
+        self.G._del_edges.extend(list(self.G.out_edges(nbunch=broken_nodes,
+            data=True)))
 
         for n in broken_nodes:
             self.damaged_areas.add(self.G.area[n])
             self.G.remove_node(n)
 
     def apply_perturbation(self, perturbed_nodes, params, weights, parallel,
-        kind='element'):
+        verbose, origin, kind='element'):
         """
 
         Perturbation simulator, actually applying the perturbation
@@ -523,12 +562,19 @@ class FaultDiagnosis():
         :param dict weights: weights for fitness evaluation on individuals.
             Dict of: {str: float, str: float, str: float}:
             - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+            - 'w2': weight multiplying total final service (default to -1.0)
+            - 'w3': weight multiplying final graph size (default to -1.0)
+            - 'w4': weight multiplying number of users with non-zero service
+                (default to -1.0)
+            - 'w5': weight for service balance over users (default to 2.0)
         :param bool parallel: flag for parallel fitness evaluation of
-            initial population
+            initial population.
+        :param bool verbose: flag for verbose printing.
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
         :param str kind: type of simulation, used to label output files,
-            default to 'element'
+            default to 'element'.
 
         .. note:: A perturbation, depending on the considered system,
             may spread in all directions starting from the damaged
@@ -536,10 +582,20 @@ class FaultDiagnosis():
         """
 
         if self.G.switches:
-            res = np.array(self.optimizer(perturbed_nodes, self.G.init_status,
-                params, weights, parallel))
-            best = dict(zip(self.G.init_status.keys(),
-                res[np.argmin(res[:, 1]), 0]))
+
+            res = self.optimizer(perturbed_nodes, self.G.init_status, params,
+                weights, parallel, origin)
+            w = np.asarray(list(weights.values()))
+
+            if verbose:
+                with open('results_generations.dat', 'w', encoding='utf8') as f:
+                    for r in res:
+                        f.write(f'{r[0]} {r[1]}')
+                        fit = np.dot(w, np.asarray(r[1]))
+                        f.write(f'{fit}\n')
+
+            res.sort(key=lambda x: np.dot(w, np.asarray(x[1])))
+            best = dict(zip(self.G.init_status.keys(), res[0][0]))
 
             initial_condition_sw = list(self.G.init_status.values())
             final_condition_sw = list(best.values())
@@ -549,10 +605,12 @@ class FaultDiagnosis():
         init_open_edges = {}
         for sw, closed in self.G.init_status.items():
             if not closed:
-                logging.debug(f'Opened switch {sw} in initial configuration')
+                logging.debug('Opened switch %s in initial configuration', sw)
+                init_open_edges[sw] = {}
                 for pred in list(self.G.predecessors(sw)):
                     # if final config is closed for this switch I memorize it
-                    if flips[sw]: init_open_edges[sw] = {pred: self.G[pred][sw]}
+                    if flips[sw]:
+                        init_open_edges[sw][pred] = self.G[pred][sw]
                     self.G.remove_edge(pred, sw)
 
         self.check_paths_and_measures(prefix='original_')
@@ -564,24 +622,33 @@ class FaultDiagnosis():
             'outdegree_centrality', 'degree_centrality'])
 
         if self.G.switches:
-            for sw, closed in best.items():
+            for sw, status in best.items():
                 if flips[sw]:
 
-                    if not closed:
-                        logging.debug(f'Switch {sw} finally open, first closed')
+                    if not status:
+                        logging.debug('Switch %s finally False, first True',
+                        sw)
                         for pre in list(self.G.predecessors(sw)):
                             self.G.remove_edge(pre, sw)
 
                     else:
-                        logging.debug(f'Switch {sw} finally closed, first open')
+                        logging.debug('Switch %s finally True, first False',
+                        sw)
                         for pre, attrs in init_open_edges[sw].items():
                             self.G.add_edge(pre, sw, **attrs)
 
-            logging.debug(f'BEST: {best}, with fitness: {np.min(res[:, 1])}')
+            print(f'\nBEST: {best}')
+            print(f'Number of actions: {res[0][1][0]}')
+            print(f'Total final service: {res[0][1][1]}')
+            print(f'Number of survived nodes: {res[0][1][2]}')
+            print(f'Number of users with non-zero service: {res[0][1][3]}')
+            print(f'Distance from average: {res[0][1][4]}')
+            fit = np.dot(w, np.asarray(res[0][1]))
             self.G.final_status = best
 
         for node in perturbed_nodes:
-            if node in self.G.nodes(): self.delete_a_node(node)
+            if node in self.G.nodes():
+                self.delete_a_node(node, origin)
 
         self.check_paths_and_measures(prefix='final_')
         self.paths_df.to_csv('service_paths_' + str(kind)+ '_perturbation.csv',
@@ -595,7 +662,8 @@ class FaultDiagnosis():
 
     def simulate_element_perturbation(self, perturbed_nodes,
         params={'npop': 300, 'ngen': 100, 'indpb': 0.6, 'tresh': 0.5,
-        'nsel': 5}, weights={'w1': 1.0, 'w2': 1.0, 'w3': 1.0}, parallel=False):
+        'nsel': 5}, weights={'w1': 1.0, 'w2': -1.0, 'w3': -1.0, 'w4': -1.0,
+        'w5': 2.0}, parallel=False, verbose=True, origin=None):
         """
 
         Simulate a perturbation of one or multiple nodes.
@@ -614,12 +682,20 @@ class FaultDiagnosis():
         :param weights: weights for fitness evaluation on individuals.
             Dict of: {str: float, str: float, str: float}:
             - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+            - 'w2': weight multiplying total final service (default to -1.0)
+            - 'w3': weight multiplying final graph size (default to -1.0)
+            - 'w4': weight multiplying number of users with non-zero service
+                (default to -1.0)
+            - 'w5': weight for service balance over users (default to 2.0)
         :type weights: dict, optional
         :param parallel: flag for parallel fitness evaluation of
-            initial population, default to False
+            initial population, default to False.
         :type parallel: bool, optional
+        :param verbose: flag for verbose output, default to True.
+        :type verbose: bool, optional
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
 
         .. note:: A perturbation, depending on the considered system,
             may spread in all directions starting from the damaged
@@ -631,17 +707,18 @@ class FaultDiagnosis():
         for node in perturbed_nodes:
 
             if node not in self.G.nodes():
-                logging.debug(f'The node {node} is not in the graph')
+                logging.debug('The node %s is not in the graph', node)
                 logging.debug('Insert a valid node')
-                logging.debug(f'Valid nodes: {self.G.nodes()}')
+                logging.debug('Valid nodes: %s', self.G.nodes())
                 sys.exit()
 
         self.apply_perturbation(perturbed_nodes, params, weights, parallel,
-            kind='element')
+            verbose, origin, kind='element')
 
     def simulate_area_perturbation(self, perturbed_areas, params={'npop': 300,
-        'ngen': 100, 'indpb': 0.6, 'tresh': 0.5, 'nsel': 5}, weights={'w1': 1.0,
-        'w2': 1.0, 'w3': 1.0}, parallel=False):
+        'ngen': 100, 'indpb': 0.6, 'tresh': 0.5, 'nsel': 5},
+        weights={'w1': 1.0, 'w2': -1.0, 'w3': -1.0, 'w4': -1.0, 'w5': 2.0},
+        parallel=False, verbose=True, origin=None):
         """
 
         Simulate a perturbation in one or multiple areas.
@@ -652,7 +729,7 @@ class FaultDiagnosis():
             - 'npop': number of individuals for each population (default to 300)
             - 'ngen': total number of generations (default to 100)
             - 'indpb': independent probability for attributes to be changed
-            (default to 0.6) 
+            (default to 0.6)
             - 'tresh' threshold for applying crossover/mutation
             (default to 0.5)
             - 'nsel': number of individuals to select (default to 5)
@@ -660,12 +737,18 @@ class FaultDiagnosis():
         :param weights: weights for fitness evaluation on individuals.
             Dict of: {str: float, str: float, str: float}:
             - 'w1': weight multiplying number of actions (default to 1.0)
-            - 'w2': weight multiplying total final service (default to 1.0)
-            - 'w3': weight multiplying final graph size (default to 1.0)
+            - 'w2': weight multiplying total final service (default to -1.0)
+            - 'w3': weight multiplying final graph size (default to -1.0)
+            - 'w4': weight multiplying number of users with non-zero service
+                (default to -1.0)
+            - 'w5': weight for service balance over users (default to 2.0)
         :type weights: dict, optional
         :param parallel: flag for parallel fitness evaluation of
-            initial population, default to False
+            initial population, default to False.
         :type parallel: bool, optional
+        :param origin: origin of the perturbation. Nodes can be resitant to
+            different kinds of perturbations, default to None.
+        :type origin: string, optional
 
         .. note:: A perturbation, depending on the considered system,
             may spread in all directions starting from the damaged
@@ -679,16 +762,17 @@ class FaultDiagnosis():
         for area in perturbed_areas:
 
             if area not in list(self.G.area.values()):
-                logging.debug(f'The area {area} is not in the graph')
+                logging.debug('The area %s is not in the graph', area)
                 logging.debug('Insert a valid area')
-                logging.debug(f'Valid areas: {set(self.G.area.values())}')
+                logging.debug('Valid areas: %s', set(self.G.area.values()))
                 sys.exit()
             else:
                 for idx, idx_area in self.G.area.items():
-                    if idx_area == area: nodes_in_area.append(idx)
+                    if idx_area == area:
+                        nodes_in_area.append(idx)
 
         self.apply_perturbation(nodes_in_area, params, weights, parallel,
-            kind='area')
+            verbose, origin, kind='area')
 
     def graph_characterization_to_file(self, filename):
         """
