@@ -9,7 +9,6 @@ import networkx as nx
 import pandas as pd
 from matplotlib import cm
 from matplotlib.colors import ListedColormap
-import matplotlib.patches as mpatches
 import matplotlib.lines as mlines
 import matplotlib.pyplot as plt
 
@@ -30,6 +29,8 @@ class GeneralGraph(nx.DiGraph):
 
     def __init__(self):
         super().__init__()
+        self._del_nodes = []
+        self._del_edges = []
 
     def load(self, filename):
         """
@@ -47,13 +48,12 @@ class GeneralGraph(nx.DiGraph):
         :rtype: pandas.DataFrame, pandas.DataFrame
         """
 
-        conv = {'mark' : str, 'father_mark' : str, }
+        conv = {'mark' : str, 'father_mark' : str,
+            'perturbation_resistant' : str}
         graph_df = pd.read_csv(filename, converters=conv, keep_default_na=False)
-        cols_to_num = ['perturbation_resistant', 'init_status']
+        cols_to_num = ['init_status']
         graph_df[cols_to_num] = graph_df[cols_to_num].apply(pd.to_numeric,
             errors='coerce', axis=1)
-        cols_to_int = ['perturbation_resistant']
-        graph_df[cols_to_int] = graph_df[cols_to_int].astype('int64')
 
         for index, row in graph_df.iterrows():
 
@@ -77,8 +77,15 @@ class GeneralGraph(nx.DiGraph):
         graph_df.set_index('mark', inplace=True)
 
         self._final_status = {}
-        nx.set_node_attributes(self, 'AVAILABLE', 'status_area')
-        nx.set_node_attributes(self, 'ACTIVE', 'mark_status')
+        nx.set_node_attributes(self, 'AVAILABLE', name='status_area')
+        nx.set_node_attributes(self, 'ACTIVE', name='mark_status')
+
+        perts = nx.get_node_attributes(self, 'perturbation_resistant')
+        split_perts = {node: perts[node].split("/")
+            for node in self.nodes}
+        strip_perts = {node: [x.strip() for x in split_perts[node]]
+            for node in self.nodes}
+        nx.set_node_attributes(self, strip_perts, name='perturbation_resistant')
 
         return graph_df, graph_edges_df
 
@@ -278,7 +285,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         computed_service = nx.get_node_attributes(self, 'computed_service')
-        if computed_service: return computed_service
+        if computed_service:
+            return computed_service
 
         computed_service, _ = self.compute_service()
         nx.set_node_attributes(self, computed_service, name='computed_service')
@@ -299,7 +307,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         shortest_path = nx.get_node_attributes(self, 'shortest_path')
-        if shortest_path: return shortest_path
+        if shortest_path:
+            return shortest_path
 
         shortest_path, shortest_path_length = self.calculate_shortest_path()
         nx.set_node_attributes(self, shortest_path, name='shortest_path')
@@ -321,7 +330,8 @@ class GeneralGraph(nx.DiGraph):
 
         shortest_path_length = nx.get_node_attributes(self,
             'shortest_path_length')
-        if shortest_path_length: return shortest_path_length
+        if shortest_path_length:
+            return shortest_path_length
 
         shortest_path, shortest_path_length = self.calculate_shortest_path()
         nx.set_node_attributes(self, shortest_path, name='shortest_path')
@@ -344,7 +354,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         efficiency = nx.get_node_attributes(self, 'efficiency')
-        if efficiency: return efficiency
+        if efficiency:
+            return efficiency
 
         efficiency = self.compute_efficiency()
         nx.set_node_attributes(self, efficiency, name='efficiency')
@@ -363,7 +374,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         nodal_efficiency = nx.get_node_attributes(self, 'nodal_efficiency')
-        if nodal_efficiency: return nodal_efficiency
+        if nodal_efficiency:
+            return nodal_efficiency
 
         nodal_efficiency = self.compute_nodal_efficiency()
         nx.set_node_attributes(self, nodal_efficiency, name='nodal_efficiency')
@@ -382,7 +394,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         local_efficiency = nx.get_node_attributes(self, 'local_efficiency')
-        if local_efficiency: return local_efficiency
+        if local_efficiency:
+            return local_efficiency
 
         local_efficiency = self.compute_local_efficiency()
         nx.set_node_attributes(self, local_efficiency, name='local_efficiency')
@@ -423,7 +436,8 @@ class GeneralGraph(nx.DiGraph):
 
         betweenness_centrality = nx.get_node_attributes(self,
             'betweenness_centrality')
-        if betweenness_centrality: return betweenness_centrality
+        if betweenness_centrality:
+            return betweenness_centrality
 
         betweenness_centrality = self.compute_betweenness_centrality()
         nx.set_node_attributes(self, betweenness_centrality,
@@ -444,7 +458,8 @@ class GeneralGraph(nx.DiGraph):
 
         closeness_centrality = nx.get_node_attributes(self,
             'closeness_centrality')
-        if closeness_centrality: return closeness_centrality
+        if closeness_centrality:
+            return closeness_centrality
 
         closeness_centrality = self.compute_closeness_centrality()
         nx.set_node_attributes(self, closeness_centrality,
@@ -464,7 +479,8 @@ class GeneralGraph(nx.DiGraph):
         """
 
         degree_centrality = nx.get_node_attributes(self, 'degree_centrality')
-        if degree_centrality: return degree_centrality
+        if degree_centrality:
+            return degree_centrality
 
         degree_centrality = self.compute_degree_centrality()
         nx.set_node_attributes(self, degree_centrality,
@@ -485,7 +501,8 @@ class GeneralGraph(nx.DiGraph):
 
         indegree_centrality = nx.get_node_attributes(self,
             'indegree_centrality')
-        if indegree_centrality: return indegree_centrality
+        if indegree_centrality:
+            return indegree_centrality
 
         indegree_centrality = self.compute_indegree_centrality()
         nx.set_node_attributes(self, indegree_centrality,
@@ -506,7 +523,8 @@ class GeneralGraph(nx.DiGraph):
 
         outdegree_centrality = nx.get_node_attributes(self,
             'outdegree_centrality')
-        if outdegree_centrality: return outdegree_centrality
+        if outdegree_centrality:
+            return outdegree_centrality
 
         outdegree_centrality = self.compute_outdegree_centrality()
         nx.set_node_attributes(self, outdegree_centrality,
@@ -550,7 +568,7 @@ class GeneralGraph(nx.DiGraph):
         shortest_paths = {}
 
         for i in nodes:
-            
+
             all_targets_paths = {}
 
             for j in sorted(list(self.H)):
@@ -624,7 +642,8 @@ class GeneralGraph(nx.DiGraph):
         :param distance: matrix of distances.
         :type distance: numpy.ndarray or multiprocessing.sharedctypes.RawArray
         :param predecessor: matrix of predecessors.
-        :type predecessor: numpy.ndarray or multiprocessing.sharedctypes.RawArray
+        :type predecessor: numpy.ndarray or
+            multiprocessing.sharedctypes.RawArray
         :param int init: starting column of numpy matrix slice.
         :param int stop: ending column of numpy matrix slice.
         :param multiprocessing.synchronize.Barrier barrier:
@@ -693,7 +712,7 @@ class GeneralGraph(nx.DiGraph):
                                    self.ids_reversed[value[-1]]]
                 shortest_path_length[self.ids[i]][key] =  length_path
 
-        return nonempty_shortest_path, shortest_path_length 
+        return nonempty_shortest_path, shortest_path_length
 
     def dijkstra_single_source_shortest_path(self):
         """
@@ -747,12 +766,12 @@ class GeneralGraph(nx.DiGraph):
         n_of_nodes = self.order()
         graph_density = nx.density(self)
 
-        logging.debug(f'In the graph are present {n_of_nodes} nodes')
+        logging.debug('In the graph are present %s nodes', n_of_nodes)
         if graph_density <= 0.000001:
-            logging.debug(f'The graph is sparse, density = {graph_density}')
+            logging.debug('The graph is sparse, density = %s', graph_density)
             shpath, shpath_len = self.dijkstra_single_source_shortest_path()
         else:
-            logging.debug(f'The graph is dense, density = {graph_density}')
+            logging.debug('The graph is dense, density = %s', graph_density)
             shpath, shpath_len = self.floyd_warshall_predecessor_and_distance()
 
         return shpath, shpath_len
@@ -898,7 +917,7 @@ class GeneralGraph(nx.DiGraph):
             when v is removed. Equivalently, local efficiency measures
             the resilience of the digraph to the perturbation of node removal,
             i.e. if we remove a node, how efficiently its first-order outgoing
-            neighbors can communicate. 
+            neighbors can communicate.
             It is in the range [0, 1].
 
         :return: local efficiency computed for every node.
@@ -925,7 +944,7 @@ class GeneralGraph(nx.DiGraph):
         :rtype: list
         """
 
-        tot_shortest_paths_list = list()
+        tot_shortest_paths_list = []
 
         for n in nodes:
             node_tot_shortest_paths = shortest_path[n]
@@ -938,7 +957,7 @@ class GeneralGraph(nx.DiGraph):
     def betweenness_centrality_kernel(self, nodes, tot_shortest_paths_list):
         """
 
-        Compute betweenness centrality, from shortest path list. 
+        Compute betweenness centrality, from shortest path list.
 
         :param list nodes: list of nodes for which to compute the
             efficiency between them and all the other nodes.
@@ -991,7 +1010,7 @@ class GeneralGraph(nx.DiGraph):
         tot_shortest_paths_list, graph_size):
         """
 
-        Compute betweenness centrality, from shortest path list. 
+        Compute betweenness centrality, from shortest path list.
 
         :param list nodes: list of nodes for which to compute the
             efficiency between them and all the other nodes.
@@ -1191,8 +1210,7 @@ class GeneralGraph(nx.DiGraph):
     def compute_service(self):
         """
 
-        Compute service for every node,
-        together with edge splitting.
+        Compute service for every node, together with edge splitting.
 
         :return: computed service computed for every node;
             splitting computed for every edge.
@@ -1233,9 +1251,9 @@ class GeneralGraph(nx.DiGraph):
 
     def print_graph(self, radius=None, initial_pos=None, fixed_nodes=None,
         n_iter=500, thresh=0.0001, size=800, border='black', edge_width=1.0,
-        arrow_size=10, fsize=12, fcolor='k', ffamily='sans-serif',
-        title='Graph', input_cmap=None, legend=True, legend_loc='upper right',
-        legend_ncol=1, legend_anchor=None, legend_fsize=12, save_to_file=None):
+        arrow_size=10, fsize=12, fcolor='k', title='Graph', input_cmap=None,
+        legend=True, legend_loc='upper right', legend_ncol=1,
+        legend_anchor=None, legend_fsize=12, save_to_file=None, status=None):
         """
 
         Print the graph.
@@ -1259,58 +1277,65 @@ class GeneralGraph(nx.DiGraph):
         :type radius: float, optional, default to 1/sqrt(n) where n is the
             number of nodes in the graph
         :param initial_pos: initial positions for nodes as a dictionary with
-            node as keys and values as a coordinate list or tuple. If None,
-            then use random initial positions.
-        :type initial_pos: dict, optional, default to None
-        :param fixed_nodes: nodes to keep fixed at initial position. ValueError
-            raised if `fixed_nodes` specified and `initial_pos` not.
-        :type fixed_nodes: list, optional, default to None
+            node as keys and values as a coordinate list or tuple, default to
+            None. If None, then use random initial positions.
+        :type initial_pos: dict, optional
+        :param fixed_nodes: nodes to keep fixed at initial position, default to
+            None. ValueError raised if `fixed_nodes` specified and
+            `initial_pos` not.
+        :type fixed_nodes: list, optional
         :param n_iter: maximum number of iterations taken in spring layout
-            simulation.
-        :type n_iter: int, optional, default to 500
+            simulation, default to 500.
+        :type n_iter: int, optional
         :param thresh: threshold for relative error in node position changes.
-            The iteration stops if the error is below this threshold.
-        :type thresh: float, optional, default to 0.0001
-        :param size: size of nodes.
-        :type size: int, optional, default to 800
-        :param border: color of node borders.
-        :type border: color, optional, default to 'black'
-        :param edge_width: width of edges.
-        :type edge_width: float, optional, default to 1.0
-        :param arrow_size: size of the arrow head head’s length and width.
-        :type arrow_size: int, optional, default to 10
-        :param fsize: font size for text labels.
-        :type fsize: int, optional, default to 12
-        :param fcolor: font color string for labels.
-        :type fcolor: string, optional, default to 'k' (black)
-        :param ffamily: font family for labels.
-        :type ffamily: string, optional, default to 'sans-serif'
-        :param title: title for figure window.
-        :type title: string, optional, defaut to 'Graph'
+            The iteration stops if the error is below this threshold,
+            default to 0.0001.
+        :type thresh: float, optional
+        :param size: size of nodes, default to 800.
+        :type size: int, optional
+        :param border: color of node borders, default to 'black'.
+        :type border: color, optional
+        :param edge_width: width of edges, default to 1.0.
+        :type edge_width: float, optional
+        :param arrow_size: size of the arrow head’s length and width,
+            default to 10.
+        :type arrow_size: int, optional
+        :param fsize: font size for text labels, default to 12.
+        :type fsize: int, optional
+        :param fcolor: font color string for labels, default to 'k' (black).
+        :type fcolor: string, optional
+        :param title: title for figure window, default to 'Graph'.
+        :type title: string, optional
         :param input_cmap: colormap for coloring the different areas with
-            different colors. If None, all nodes are colored as white.
-        :type input_cmap: Matplotlib colormap, optional, default to None
-        :param legend: show the legend on/off.
-        :type legend: bool, optional, default to True
-        :param legend_loc: the location of the legend.
-        :type legend_loc: str, optional, default to 'upper right'
-        :param legend_ncol: the number of columns that the legend has.
-        :type legend_ncol: int, optional, default to 1
+            different colors, default to None.
+            If None, all nodes are colored as white.
+        :type input_cmap: Matplotlib colormap
+        :param legend: show the legend on/off, default to True.
+        :type legend: bool, optional
+        :param legend_loc: the location of the legend, default to 'upper right'.
+        :type legend_loc: str, optional
+        :param legend_ncol: the number of columns that the legend has,
+            default to 1.
+        :type legend_ncol: int, optional
         :param legend_anchor: box that is used to position the legend in
-            conjunction with loc.
-        :type legend_anchor: 2-tuple, or 4-tuple of floats, optional,
-            defaults to axes.bbox (if called as a method to Axes.legend)
-            or figure.bbox (if Figure.legend).
-            This argument allows arbitrary placement of the legend
+            conjunction with loc, defaults to axes.bbox (if called as a method
+            to Axes.legend) or figure.bbox (if Figure.legend). This argument
+            allows arbitrary placement of the legend.
+        :type legend_anchor: 2-tuple, or 4-tuple of floats, optional
         :param legend_fsize: the font size of the legend. The value must be
-            numeric, implying the size the absolute font size in points.
-        :type legend_fsize: int, optional, default to 12
-        :param save_to_file: name of the file where to save the graph drawing.
-            The extension is guesses from the filename.
+            numeric, implying the size the absolute font size in points,
+            default to 12.
+        :type legend_fsize: int, optional
+        :param save_to_file: name of the file where to save the graph drawing,
+            default to None. The extension is guesses from the filename.
             Interactive window is rendered in any case.
-        :type save_to_file: string, optional, default to None
-
+        :type save_to_file: string, optional
+        :param status: include initial condition of switches in printed graph.
+            To activate it, set it to 'initial', default to None.
+        :type status: string, optional
 
+        :return: a dictionary of positions keyed by node.
+        :rtype: dict
         :raises: ValueError
         """
 
@@ -1318,7 +1343,12 @@ class GeneralGraph(nx.DiGraph):
             raise ValueError('Fixed requested without given initial positions')
         logging.getLogger().setLevel(logging.INFO)
 
-        pos = nx.spring_layout(self, k=radius, pos=initial_pos,
+        T = copy.deepcopy(self)
+
+        T.add_nodes_from(self._del_nodes)
+        T.add_edges_from(self._del_edges)
+
+        pos = nx.spring_layout(T, k=radius, pos=initial_pos,
             fixed=fixed_nodes, iterations=n_iter, threshold=thresh,
             seed=3113794652)
 
@@ -1330,25 +1360,34 @@ class GeneralGraph(nx.DiGraph):
         else:
             mymap = cm.get_cmap(input_cmap, len(all_areas))
         area_indices = {}
-        for idx in range(len(all_areas)):
-            area_indices[all_areas[idx]] = idx*(1./len(all_areas))
+        for idx, item in enumerate(all_areas):
+            area_indices[item] = idx*(1./len(all_areas))
 
         for node in self:
             col = mymap(area_indices[self.area[node]])
             col = np.array([col])
-            nx.draw_networkx_nodes(self, pos, nodelist=[node], node_color=col,
+            nx.draw_networkx_nodes(T, pos, nodelist=[node], node_color=col,
                 node_shape=shapes[self.type[node]], node_size=size,
-                edgecolors=border)
+                edgecolors=border, alpha=0.5)
 
         pert_resistant = [node for node in self.perturbation_resistant.keys()
-             if self.perturbation_resistant[node] == 1]
+             if self.perturbation_resistant[node][0]]
 
         for node in pert_resistant:
             col = mymap(area_indices[self.area[node]])
             col = np.array([col])
-            nx.draw_networkx_nodes(self, pos, nodelist=[node], node_color=col,
+            nx.draw_networkx_nodes(T, pos, nodelist=[node], node_color=col,
                 node_shape=shapes[self.type[node]], node_size=size,
-                edgecolors='red')
+                edgecolors=border, alpha=1.0)
+
+        if self._del_nodes:
+            del_marks = {n: n for n in self._del_nodes}
+            for node in self._del_nodes:
+                nx.draw_networkx_nodes(T, pos, nodelist=[node],
+                    node_color='white', node_shape='.',
+                    node_size=size, edgecolors='white', alpha=1.0)
+            nx.draw_networkx_labels(T, pos, labels=del_marks,
+                font_size=fsize, font_color='red')
 
         or_edges = [(u, v) for (u, v, d) in self.edges(data=True)
             if d['father_condition'] == 'OR']
@@ -1357,17 +1396,46 @@ class GeneralGraph(nx.DiGraph):
         single_edges = [(u, v) for (u, v, d) in self.edges(data=True)
             if d['father_condition'] == 'SINGLE']
 
-        nx.draw_networkx_edges(self, pos, edgelist=or_edges, width=edge_width,
+        #deleted edges
+        or_edges_del = [(u, v) for (u, v, d) in self._del_edges
+            if d['father_condition'] == 'OR']
+        and_edges_del = [(u, v) for (u, v, d) in self._del_edges
+            if d['father_condition'] == 'AND']
+        single_edges_del = [(u, v) for (u, v, d) in self._del_edges
+            if d['father_condition'] == 'SINGLE']
+
+        if status == 'initial':
+            open_sw = [k for k, v in self.init_status.items() if not v]
+            edges_no_init = []
+            for sw in open_sw:
+                for pred in list(self.predecessors(sw)):
+                    edges_no_init.append((pred,sw))
+
+            or_edges = list(set(or_edges)-set(edges_no_init))
+            and_edges = list(set(and_edges)-set(edges_no_init))
+            single_edges = list(set(single_edges)-set(edges_no_init))
+
+        nx.draw_networkx_edges(T, pos, edgelist=or_edges, width=edge_width,
             arrowsize=arrow_size, alpha=0.9, edge_color='brown', style='dashed',
             node_size=size)
-        nx.draw_networkx_edges(self, pos, edgelist=and_edges, width=edge_width,
+        nx.draw_networkx_edges(T, pos, edgelist=and_edges, width=edge_width,
             arrowsize=arrow_size, alpha=0.9, edge_color='violet',
             node_size=size)
-        nx.draw_networkx_edges(self, pos, edgelist=single_edges,
+        nx.draw_networkx_edges(T, pos, edgelist=single_edges,
             width=edge_width, arrowsize=arrow_size, alpha=0.9,
             edge_color='black', node_size=size)
 
-        nx.draw_networkx_labels(self, pos, labels=self.mark, font_size=fsize,
+        nx.draw_networkx_edges(T, pos, edgelist=or_edges_del,
+            width=(edge_width/2.), arrowsize=arrow_size, alpha=0.9,
+            edge_color='red', style='dashed', node_size=size)
+        nx.draw_networkx_edges(T, pos, edgelist=and_edges_del,
+            width=(edge_width/2.), arrowsize=arrow_size, alpha=0.9,
+            edge_color='red', node_size=size)
+        nx.draw_networkx_edges(T, pos, edgelist=single_edges_del,
+            width=(edge_width/2.), arrowsize=arrow_size, alpha=0.9,
+            edge_color='red', node_size=size)
+
+        nx.draw_networkx_labels(T, pos, labels=self.mark, font_size=fsize,
             font_color=fcolor)
 
         plt.get_current_fig_manager().canvas.set_window_title(title)
@@ -1377,22 +1445,30 @@ class GeneralGraph(nx.DiGraph):
         handles = []
         plt.rcParams.update({"text.usetex": True})
 
-        source_key = mlines.Line2D([], [], color='white', marker='v',
-            markeredgecolor='black', markersize=legend_fsize, label='SOURCE')
+        source_key = mlines.Line2D([], [], color='black', alpha=0.5, marker='v',
+            markeredgecolor='black', markersize=legend_fsize, linewidth=0.0,
+            label='SOURCE')
         handles.append(source_key)
-        user_key = mlines.Line2D([], [], color='white', marker='^',
-            markeredgecolor='black', markersize=legend_fsize, label='USER')
+        user_key = mlines.Line2D([], [], color='black', alpha=0.5, marker='^',
+            markeredgecolor='black', markersize=legend_fsize, linewidth=0.0,
+            label='USER')
         handles.append(user_key)
-        hub_key = mlines.Line2D([], [], color='white', marker='o',
-            markeredgecolor='black', markersize=legend_fsize, label='HUB')
+        hub_key = mlines.Line2D([], [], color='black', alpha=0.5, marker='o',
+            markeredgecolor='black', markersize=legend_fsize, linewidth=0.0,
+            label='HUB')
         handles.append(hub_key)
-        switch_key = mlines.Line2D([], [], color='white', marker='X',
-            markeredgecolor='black', markersize=legend_fsize, label='SWITCH')
+        switch_key = mlines.Line2D([], [], color='black', alpha=0.5, marker='X',
+            markeredgecolor='black', markersize=legend_fsize, linewidth=0.0,
+            label='SWITCH')
         handles.append(switch_key)
-        pr_key = mlines.Line2D([], [], color='white', marker='o',
-            markeredgecolor='red', markersize=legend_fsize,
+        pr_key = mlines.Line2D([], [], color='black', alpha=1.0, marker='o',
+            markeredgecolor='black', markersize=legend_fsize, linewidth=0.0,
             label='Perturbation Resistant')
         handles.append(pr_key)
+        del_node_key = mlines.Line2D([], [], color='red', alpha=1.0, marker='.',
+            markeredgecolor='white', markersize=legend_fsize, linewidth=0.0,
+            label='Broken node')
+        handles.append(del_node_key)
 
         single_key = mlines.Line2D([], [], color='white',
             marker=r'$\rightarrow$', markeredgecolor='black',
@@ -1406,12 +1482,18 @@ class GeneralGraph(nx.DiGraph):
             marker=r'$\rightarrow$', markeredgecolor='violet',
             markersize=legend_fsize, label='AND')
         handles.append(and_key)
+        del_edge_key = mlines.Line2D([], [], color='white',
+            marker=r'$\rightarrow$', markeredgecolor='red',
+            markersize=legend_fsize, label='Broken edge')
+        handles.append(del_edge_key)
 
         if legend:
             plt.legend(handles=handles, loc=legend_loc, ncol=legend_ncol,
-            bbox_to_anchor=legend_anchor, fontsize=legend_fsize)        
+            bbox_to_anchor=legend_anchor, fontsize=legend_fsize)
 
         if save_to_file:
             plt.savefig(save_to_file, orientation='landscape', transparent=True)
         else:
             plt.show()
+
+        return pos
